@@ -19,6 +19,8 @@ TEST_MODULES=(
     "dkms_failing_test"
     "dkms_dependencies_test"
     "dkms_multiver_test"
+    "dkms_nover_test"
+    "dkms_emptyver_test"
 )
 TEST_TMPDIRS=(
     "/usr/src/dkms_test-1.0/"
@@ -26,6 +28,8 @@ TEST_TMPDIRS=(
     "/usr/src/dkms_dependencies_test-1.0"
     "/usr/src/dkms_multiver_test-1.0"
     "/usr/src/dkms_multiver_test-2.0"
+    "/usr/src/dkms_nover_test-1.0"
+    "/usr/src/dkms_emptyver_test-1.0"
     "/tmp/dkms_test_dir_${KERNEL_VER}/"
 )
 TEST_TMPFILES=(
@@ -59,6 +63,8 @@ clean_dkms_env() {
         if [[ -n "$found_module" ]] ; then
             dkms remove ${module}/1.0 >/dev/null
         fi
+        rm -rf "/var/lib/dkms/${module}/"
+        rm -f "/lib/modules/${KERNEL_VER}/${expected_dest_loc}/${module}.ko${mod_compression_ext}"
     done
     for dir in "${TEST_TMPDIRS[@]}"; do
         rm -rf "$dir"
@@ -77,6 +83,14 @@ check_no_dkms_test() {
             echo >&2 "Error: module ${module} is still in DKMS tree"
             exit 1
         fi
+        if [[ -d "/var/lib/dkms/${module}" ]]; then
+            echo >&2 "Error: directory /var/lib/dkms/${module} still exists"
+            exit 1
+        fi
+        if [[ -f "/lib/modules/${KERNEL_VER}/${expected_dest_loc}/${module}.ko${mod_compression_ext}" ]]; then
+            echo >&2 "Error: file /lib/modules/${KERNEL_VER}/${expected_dest_loc}/${module}.ko${mod_compression_ext} still exists"
+            exit 1
+        fi
     done
     for dir in "${TEST_TMPDIRS[@]}"; do
         if [[ -d "$dir" ]]; then
@@ -133,30 +147,31 @@ genericize_expected_output() {
     local output_log=$1
 
     # "depmod..." lines can have multiple points. Replace them, to be able to compare
-    sed 's/\([^.]\)\.\.\.\.*$/\1.../' -i ${output_log}
+    sed -i 's/\([^.]\)\.\.\.\.*$/\1.../' ${output_log}
     # On CentOS, weak-modules is executed. Drop it from the output, to be more generic
-    sed '/^Adding any weak-modules$/d' -i ${output_log}
-    sed '/^Removing any linked weak-modules$/d' -i ${output_log}
+    sed -i '/^Adding any weak-modules$/d' ${output_log}
+    sed -i '/^Removing any linked weak-modules$/d' ${output_log}
     # "depmod..." lines are missing when uninstalling modules on CentOS. Remove them to be more generic
     if [[ $# -ge 2 && "$2" =~ uninstall|unbuild|remove ]] ; then
-        sed '/^depmod\.\.\.$/d' -i ${output_log}
+        sed -i '/^depmod\.\.\.$/d' ${output_log}
     fi
     # Signing related output. Drop it from the output, to be more generic
-    sed '/^Sign command:/d' -i ${output_log}
-    sed '/^Signing key:/d' -i ${output_log}
-    sed '/^Public certificate (MOK):/d' -i ${output_log}
-    sed '/^Certificate or key are missing, generating them using update-secureboot-policy...$/d' -i ${output_log}
-    sed '/^Certificate or key are missing, generating self signed certificate for MOK...$/d' -i ${output_log}
+    sed -i '/^Sign command:/d' ${output_log}
+    sed -i '/^Signing key:/d' ${output_log}
+    sed -i '/^Public certificate (MOK):/d' ${output_log}
+    sed -i '/^Certificate or key are missing, generating them using update-secureboot-policy...$/d' ${output_log}
+    sed -i '/^Certificate or key are missing, generating self signed certificate for MOK...$/d' ${output_log}
     if [[ "${NO_SIGNING_TOOL}" = "1" ]]; then
-        sed "/^Binary .* not found, modules won't be signed$/d" -i ${output_log}
+        sed -i "/^Binary .* not found, modules won't be signed$/d" ${output_log}
         # Uncomment the following line to run this script with --no-signing-tool on platforms where the sign-file tool exists
-        # sed '/^Signing module \/var\/lib\/dkms\/dkms_test\/1.0\/build\/dkms_test.ko$/d' -i ${output_log}
+        # sed -i '/^Signing module \/var\/lib\/dkms\/dkms_test\/1.0\/build\/dkms_test.ko$/d' ${output_log}
     fi
     # OpenSSL non-critical errors while signing. Remove them to be more generic
-    sed '/^At main.c:/d' -i ${output_log}
-    sed '/^- SSL error:/d' -i ${output_log}
+    sed -i '/^At main.c:/d' ${output_log}
+    sed -i '/^- SSL error:/d' ${output_log}
     # Apport related error that can occur in the CI. Drop from the output to be more generic
-    sed "/^python3: can't open file '\/usr\/share\/apport\/package-hooks\/dkms_packages.py'\: \[Errno 2\] No such file or directory$/d" -i ${output_log}
+    sed -i "/^python3: can't open file '\/usr\/share\/apport\/package-hooks\/dkms_packages.py'\: \[Errno 2\] No such file or directory$/d" ${output_log}
+    sed -i "/^ERROR (dkms apport): /d" ${output_log}
 }
 
 run_with_expected_output() {
@@ -235,7 +250,7 @@ case "${os_id}" in
 esac
 
 
-echo 'Checking that the environment is clean again'
+echo 'Preparing a clean test environment'
 clean_dkms_env
 
 echo 'Test framework file hijacking'
@@ -818,8 +833,155 @@ EOF
 echo 'Removing /usr/src/dkms_multiver_test-1.0 /usr/src/dkms_multiver_test-2.0'
 rm -r /usr/src/dkms_multiver_test-1.0 /usr/src/dkms_multiver_test-2.0
 
-echo 'Checking that the environment is clean again'
-clean_dkms_env
+
+echo 'Adding the nover/emptyver test modules by directory'
+run_with_expected_output dkms add test/dkms_nover_test << EOF
+Creating symlink /var/lib/dkms/dkms_nover_test/1.0/source -> /usr/src/dkms_nover_test-1.0
+EOF
+run_status_with_expected_output 'dkms_nover_test' << EOF
+dkms_nover_test/1.0: added
+EOF
+if ! [[ -d /usr/src/dkms_nover_test-1.0 ]] ; then
+    echo >&2 'Error: directory /usr/src/dkms_nover_test-1.0 was not created'
+    exit 1
+fi
+run_with_expected_output dkms add test/dkms_emptyver_test << EOF
+Creating symlink /var/lib/dkms/dkms_emptyver_test/1.0/source -> /usr/src/dkms_emptyver_test-1.0
+EOF
+run_status_with_expected_output 'dkms_emptyver_test' << EOF
+dkms_emptyver_test/1.0: added
+EOF
+if ! [[ -d /usr/src/dkms_emptyver_test-1.0 ]] ; then
+    echo >&2 'Error: directory /usr/src/dkms_emptyver_test-1.0 was not created'
+    exit 1
+fi
+
+echo 'Building the nover/emptyver test modules'
+set_signing_message "dkms_nover_test" "1.0"
+run_with_expected_output dkms build -k "${KERNEL_VER}" -m dkms_nover_test -v 1.0 << EOF
+
+Building module:
+Cleaning build area...
+make -j$(nproc) KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_nover_test/1.0/build...
+${SIGNING_MESSAGE}Cleaning build area...
+EOF
+run_status_with_expected_output 'dkms_nover_test' << EOF
+dkms_nover_test/1.0, ${KERNEL_VER}, $(uname -m): built
+EOF
+set_signing_message "dkms_emptyver_test" "1.0"
+run_with_expected_output dkms build -k "${KERNEL_VER}" -m dkms_emptyver_test -v 1.0 << EOF
+
+Building module:
+Cleaning build area...
+make -j$(nproc) KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_emptyver_test/1.0/build...
+${SIGNING_MESSAGE}Cleaning build area...
+EOF
+run_status_with_expected_output 'dkms_emptyver_test' << EOF
+dkms_emptyver_test/1.0, ${KERNEL_VER}, $(uname -m): built
+EOF
+
+echo 'Installing the nover/emptyver test modules'
+run_with_expected_output dkms install -k "${KERNEL_VER}" -m dkms_nover_test -v 1.0 << EOF
+
+dkms_nover_test.ko${mod_compression_ext}:
+Running module version sanity check.
+ - Original module
+   - No original module exists within this kernel
+ - Installation
+   - Installing to /lib/modules/${KERNEL_VER}/${expected_dest_loc}/
+depmod...
+EOF
+run_status_with_expected_output 'dkms_nover_test' << EOF
+dkms_nover_test/1.0, ${KERNEL_VER}, $(uname -m): installed
+EOF
+run_with_expected_output dkms install -k "${KERNEL_VER}" -m dkms_emptyver_test -v 1.0 << EOF
+
+dkms_emptyver_test.ko${mod_compression_ext}:
+Running module version sanity check.
+ - Original module
+   - No original module exists within this kernel
+ - Installation
+   - Installing to /lib/modules/${KERNEL_VER}/${expected_dest_loc}/
+depmod...
+EOF
+run_status_with_expected_output 'dkms_emptyver_test' << EOF
+dkms_emptyver_test/1.0, ${KERNEL_VER}, $(uname -m): installed
+EOF
+
+echo 'Uninstalling the nover/emptyver test modules'
+run_with_expected_output dkms uninstall -k "${KERNEL_VER}" -m dkms_nover_test -v 1.0 << EOF
+Module dkms_nover_test-1.0 for kernel ${KERNEL_VER} ($(uname -m)).
+Before uninstall, this module version was ACTIVE on this kernel.
+
+dkms_nover_test.ko${mod_compression_ext}:
+ - Uninstallation
+   - Deleting from: /lib/modules/${KERNEL_VER}/${expected_dest_loc}/
+ - Original module
+   - No original module was found for this module on this kernel.
+   - Use the dkms install command to reinstall any previous module version.
+EOF
+run_status_with_expected_output 'dkms_nover_test' << EOF
+dkms_nover_test/1.0, ${KERNEL_VER}, $(uname -m): built
+EOF
+if [[ -e "/lib/modules/${KERNEL_VER}/${expected_dest_loc}/dkms_nover_test.ko${mod_compression_ext}" ]] ; then
+    echo >&2 "Error: module not removed in /lib/modules/${KERNEL_VER}/${expected_dest_loc}/dkms_nover_test.ko${mod_compression_ext}"
+    exit 1
+fi
+run_with_expected_output dkms uninstall -k "${KERNEL_VER}" -m dkms_emptyver_test -v 1.0 << EOF
+Module dkms_emptyver_test-1.0 for kernel ${KERNEL_VER} ($(uname -m)).
+Before uninstall, this module version was ACTIVE on this kernel.
+
+dkms_emptyver_test.ko${mod_compression_ext}:
+ - Uninstallation
+   - Deleting from: /lib/modules/${KERNEL_VER}/${expected_dest_loc}/
+ - Original module
+   - No original module was found for this module on this kernel.
+   - Use the dkms install command to reinstall any previous module version.
+EOF
+run_status_with_expected_output 'dkms_emptyver_test' << EOF
+dkms_emptyver_test/1.0, ${KERNEL_VER}, $(uname -m): built
+EOF
+if [[ -e "/lib/modules/${KERNEL_VER}/${expected_dest_loc}/dkms_emptyver_test.ko${mod_compression_ext}" ]] ; then
+    echo >&2 "Error: module not removed in /lib/modules/${KERNEL_VER}/${expected_dest_loc}/dkms_emptyver_test.ko${mod_compression_ext}"
+    exit 1
+fi
+
+echo 'Unbuilding the nover/emptyver test modules'
+run_with_expected_output dkms unbuild -k "${KERNEL_VER}" -m dkms_nover_test -v 1.0 << EOF
+Module dkms_nover_test 1.0 is not installed for kernel ${KERNEL_VER} ($(uname -m)). Skipping...
+EOF
+run_status_with_expected_output 'dkms_nover_test' << EOF
+dkms_nover_test/1.0: added
+EOF
+run_with_expected_output dkms unbuild -k "${KERNEL_VER}" -m dkms_emptyver_test -v 1.0 << EOF
+Module dkms_emptyver_test 1.0 is not installed for kernel ${KERNEL_VER} ($(uname -m)). Skipping...
+EOF
+run_status_with_expected_output 'dkms_emptyver_test' << EOF
+dkms_emptyver_test/1.0: added
+EOF
+
+echo 'Removing the nover/emptyver test modules'
+run_with_expected_output dkms remove -k "${KERNEL_VER}" -m dkms_nover_test -v 1.0 << EOF
+Module dkms_nover_test 1.0 is not installed for kernel ${KERNEL_VER} ($(uname -m)). Skipping...
+Module dkms_nover_test 1.0 is not built for kernel ${KERNEL_VER} ($(uname -m)). Skipping...
+Deleting module dkms_nover_test-1.0 completely from the DKMS tree.
+EOF
+run_status_with_expected_output 'dkms_nover_test' << EOF
+EOF
+run_with_expected_output dkms remove -k "${KERNEL_VER}" -m dkms_emptyver_test -v 1.0 << EOF
+Module dkms_emptyver_test 1.0 is not installed for kernel ${KERNEL_VER} ($(uname -m)). Skipping...
+Module dkms_emptyver_test 1.0 is not built for kernel ${KERNEL_VER} ($(uname -m)). Skipping...
+Deleting module dkms_emptyver_test-1.0 completely from the DKMS tree.
+EOF
+run_status_with_expected_output 'dkms_emptyver_test' << EOF
+EOF
+
+echo 'Removing /usr/src/dkms_nover_test-1.0 /usr/src/dkms_emptyver_test-1.0'
+rm -r /usr/src/dkms_nover_test-1.0 /usr/src/dkms_emptyver_test-1.0
+
+
+echo 'Checking that the environment is clean'
+check_no_dkms_test
 
 echo 'Running autoinstall error testing'
 
@@ -881,7 +1043,7 @@ EOF
 echo 'Removing /usr/src/dkms_dependencies_test-1.0'
 rm -r /usr/src/dkms_dependencies_test-1.0
 
-echo 'Checking that the environment is clean'
+echo 'Checking that the environment is clean again'
 check_no_dkms_test
 
 echo 'All tests successful :)'
